@@ -1,10 +1,11 @@
 # -*- coding:utf-8 -*-
 import re
 import urllib
+from urlparse import urlparse, parse_qs
 
 from pyquery import PyQuery
 import lxml.html
-from yandex_parser.exceptions import EmptySerp, YandexParserError
+from yandex_parser.exceptions import EmptySerp, YandexParserError, YandexParserContentError
 from yandex_parser.utils import to_unicode, get_full_domain_without_scheme
 from lxml import etree
 
@@ -14,8 +15,11 @@ class YandexParser(object):
 
     patterns = {
         'pagecount': re.compile(u'found"\:"&mdash;&nbsp;(.*?)отв', params_regexr),
-        'infected': re.compile(u'/search/infected/?\?url=([^&]+)', params_regexr),
-        'captcha': re.compile(u'<img class="image form__captcha".*?src=\"([^\"]+)\"', params_regexr),
+        'infected': (
+            re.compile(u'/search/infected/?\?url=([^&]+)', params_regexr),
+            re.compile(u'/safety/\?url=([^&]+)', params_regexr),
+        ),
+        'captcha': re.compile(u'<img class="image form__(?:captcha|image)".*?src=\"([^\"]+)\"', params_regexr),
     }
 
     USE_IGNORE_BLOCK = True
@@ -26,9 +30,13 @@ class YandexParser(object):
         re.compile(ur'<div class="?serp-adv__found"?>Наш[^ ]+\s+(.*?)рез', params_regexr)
     )
 
-    def __init__(self, content, snippet_fileds=('d', 'p', 'u', 't', 's', 'm')):
+    CONTEXT_ORGANIC_BLOCK = 'organic block'
+
+    def __init__(self, content, snippet_fileds=('d', 'p', 'u', 't', 's', 'm'), exclude_market_yandex=True, exclude_realty_yandex=True):
         self.content = to_unicode(content)
         self.snippet_fileds = snippet_fileds
+        self.exclude_market_yandex = exclude_market_yandex
+        self.exclude_realty_yandex = exclude_realty_yandex
 
     def get_context_snippet_title(self, content):
         res = re.search(ur'<h2[^>]*?>\s*<a[^>]*?href="([^"]+?)"[^>]*?>\s*(.*?)\s*</a>', content, re.I | re.M | re.S)
@@ -52,58 +60,175 @@ class YandexParser(object):
 
         return 'b'
 
-
     def get_context_serp(self):
-        result = []
-        is_top = True
-        old_class = None
+        # очищаем встроенные в рекламу сниппеты serp-item
+        content = re.sub(
+            ur'<div class=direct-map-modal__snippet><div class=serp-item.*?</div>\s*</div>',
+            '', self.content, flags=re.I | re.M | re.S
+        )
 
-        dom = PyQuery(self.content)
+        dom = PyQuery(content)
         serp = dom('.serp-item')
-        for sn in serp:
+
+        r_blocks, tb_blocks = self._find_context_r_or_tb_blocks(serp)
+        tb_blocks = self._aggregate_organic_blocks(tb_blocks)
+        tb_blocks = self._remove_little_organic_blocks(tb_blocks)
+        t_blocks, b_blocks = self._divide_context_tb_blocks(tb_blocks)
+
+        self._set_a(t_blocks, 't')
+        self._set_a(b_blocks, 'b')
+        self._set_a(r_blocks, 'r')
+
+        result = self._format_context_blocks(t_blocks + b_blocks + r_blocks)
+        return {'pc': len(result), 'sn': result}
+
+    def _find_context_r_or_tb_blocks(self, serp):
+        tb_blocks = []
+        r_blocks = []
+        organic_block_len = 0
+        for index, sn in enumerate(serp):
             is_ignore_block = self._ignore_block(sn)
             is_context_snippet = self._is_context_snippet(sn)
 
             if not is_ignore_block:
-                is_top = False
-                continue
-
-            if not is_context_snippet:
+                organic_block_len += 1
+                tb_blocks.append({'index': index, 'sn': self.CONTEXT_ORGANIC_BLOCK})
                 continue
 
             # исключаем блок директа с баннером
             if sn.xpath('./div[contains(@class,"composite_gap_none")]'):
                 continue
 
+            # исключаем блок директа с товарами
+            if sn.xpath('./div[contains(@class,"carousel")]'):
+                continue
+
+            # исключаем блок директа с товарами
+            if sn.xpath('./div[contains(@class,"companies-map-")]'):
+                continue
+
+            if 'data-fast-wzrd' in sn.attrib and sn.attrib['data-fast-wzrd'] == 'market_constr':
+                continue
+
+            if self._is_card_narrow(sn):
+                continue
+
+            if not is_context_snippet:
+                continue
+
             t_or_b = sn.xpath('.//div[contains(@class,"typo_type_greenurl")]/div[contains(@class,"label_color_yellow")]') \
-                or sn.cssselect('li.serp-adv-item div.organic__subtitle')
+                or sn.cssselect('li.serp-adv-item div.organic__subtitle') \
+                or sn.xpath('.//div[contains(@class,"typo_type_greenurl")]/div[contains(@class,"label_theme_direct")]')
             if t_or_b:
+                tb_blocks.append({'index': index, 'sn': sn})
+                continue
+            r_blocks.append({'index': index, 'sn': sn})
+        return r_blocks, tb_blocks
+
+    def _remove_little_organic_blocks(self, tb_blocks):
+        max_ob_len = 0
+        max_ob_index = None
+        for i, block in enumerate(tb_blocks):
+            if block['sn'] == self.CONTEXT_ORGANIC_BLOCK and block['len'] >= max_ob_len:
+                max_ob_len = block['len']
+                max_ob_index = i
+
+        if max_ob_index is None:
+            return tb_blocks
+
+        blocks = []
+        for i, block in enumerate(tb_blocks):
+            if block['sn'] != self.CONTEXT_ORGANIC_BLOCK:
+                blocks.append(block)
+
+            if i != max_ob_index:
+                continue
+
+            blocks.append(block)
+        return blocks
+
+    def _aggregate_organic_blocks(self, tb_blocks):
+        agg_blocks = []
+        ob_len = 0
+        old_ob_block = None
+        for block in tb_blocks:
+            if block['sn'] != self.CONTEXT_ORGANIC_BLOCK:
+                if ob_len:
+                    old_ob_block['len'] = ob_len
+                    agg_blocks.append(old_ob_block)
+                    old_ob_block = None
+                    ob_len = 0
+                agg_blocks.append(block)
+                continue
+
+            old_ob_block = block
+            ob_len += 1
+        if ob_len:
+            old_ob_block['len'] = ob_len
+            agg_blocks.append(old_ob_block)
+        return agg_blocks
+
+    def _divide_context_tb_blocks(self, tb_blocks):
+        t_blocks = []
+        b_blocks = []
+        is_ob_exists = filter(lambda x: x['sn'] == self.CONTEXT_ORGANIC_BLOCK, tb_blocks)
+        if is_ob_exists:
+            is_bottom = False
+            for block in tb_blocks:
+                if block['sn'] == self.CONTEXT_ORGANIC_BLOCK:
+                    is_bottom = True
+                    continue
+
+                if is_bottom:
+                    b_blocks.append(block)
+                    continue
+
+                t_blocks.append(block)
+        else:
+            is_bottom = False
+            old_class = None
+            for block in tb_blocks:
+                sn = block['sn']
                 cur_class = filter(lambda x: re.match(ur'data-[\w\d]{4,}', x), sn.attrib)[0]
                 if old_class and cur_class != old_class:
-                    is_top = False
+                    is_bottom = True
                 old_class = cur_class
 
-            if not t_or_b:
-                a = 'r'
-            elif is_top:
-                a = 't'
-            else:
-                a = 'b'
+                if is_bottom:
+                    b_blocks.append(block)
+                    continue
 
+                t_blocks.append(block)
+
+        return t_blocks, b_blocks
+
+    def _format_context_blocks(self, blocks):
+        result = []
+        for block in sorted(blocks, key=lambda x: x['index']):
+            sn = block['sn']
             title, url = self._get_title(sn, False)
-            item = {'u': url, 't': title}
-            item['vu'] = self.get_context_visible_url(sn)
-            item['a'] = a
-            result.append(item)
 
-        return {'pc': len(result), 'sn': result}
+            if not re.search('^https?://yabs\.yandex\.ru', url):
+                raise YandexParserError(u'incorrect context url={}'.format(url))
+
+            result.append({
+                'u': url,
+                't': title,
+                'vu': self.get_context_visible_url(sn),
+                'a': block['a']
+            })
+        return result
+
+    def _set_a(self, blocks, a):
+        for block in blocks:
+            block['a'] = a
 
     def get_serp(self):
         if self.is_not_found():
             return {'pc': 0, 'sn': []}
 
         if not YandexParser.is_yandex(self.content):
-            raise YandexParserError(u'content is not yandex')
+            raise YandexParserContentError(u'content is not yandex')
 
         pagecount = self.get_pagecount()
         snippets = self.get_snippets()
@@ -118,7 +243,9 @@ class YandexParser(object):
 
     @classmethod
     def is_yandex(cls, content):
-        return '<a class="logo__link" href="//www.yandex.' in content or u'<title>Яндекс' in content
+        return '<a class="logo__link" href="//www.yandex.' in content \
+            or u'<title>Яндекс' in content \
+            or 'href="https://www.yandex.ru" title="Яндекс"' in content
 
     def get_clean_html(self):
         return YandexSerpCleaner.clean(self.content)
@@ -167,6 +294,11 @@ class YandexParser(object):
         #Яндекс жжет. Берем домен из гринурла
         infected = False
 
+        # парсим турбо-страницы
+        if url.startswith('https://yandex.ru/turbo?'):
+            o = urlparse(url)
+            url = parse_qs(o.query)['text'][0]
+
         if url == 'http://':
             html = etree.tostring(sn)
             pattern = re.compile(ur'<div class="serp-item__greenurl.*?<a class="link serp-url__link".*?>(.*?)</a>', re.I | re.M | re.S)
@@ -175,13 +307,17 @@ class YandexParser(object):
                 domain = re.sub(ur'<[^>]+>', '', res.group(1), flags=re.I | re.M | re.S)
                 domain = re.sub(ur'\s+', '', domain, flags=re.I | re.M | re.S)
                 url = 'http://{}'.format(domain)
+        elif url.startswith('//'):
+            url = 'https:' + url
 
-        if 'infected' in url:
-            match_url_infected = self.patterns['infected'].match(url)
-            if match_url_infected:
-                url = urllib.unquote(match_url_infected.group(1))
+        if 'infected' in url or url.startswith('/safety/'):
+            for pattern in self.patterns['infected']:
+                res = pattern.match(url)
+                if not res:
+                    continue
+
+                url = urllib.unquote(res.group(1))
                 infected = True
-
         return url, infected
 
     def _get_title(self, sn, infected):
@@ -231,8 +367,15 @@ class YandexParser(object):
         if sn.xpath('.//div[contains(@class,"label_color_yellow")]'):
             return True
 
+        if sn.xpath('.//div[contains(@class,"label_theme_direct")]'):
+            return True
+
         return 'serp-adv' in sn.attrib['class'] or 't-construct-adapter__adv' in sn.attrib['class']
 
+    def _is_card_narrow(self, sn):
+        # боковая карта справа
+        return 'card__narrow' in sn.attrib['class']
+
     def _ignore_block(self, sn):
         if self._is_context_snippet(sn):
             return True
@@ -243,6 +386,10 @@ class YandexParser(object):
             #реклама
             return True
 
+        # боковая карта справа
+        if self._is_card_narrow(sn):
+            return True
+
         # игнорим новости
         if 't-construct-adapter__news' in sn.attrib['class']:
             return True
@@ -259,13 +406,17 @@ class YandexParser(object):
             return True
 
         # игнорим картинки
-        if 't-construct-adapter__images' in sn.attrib['class']:
+        if 't-construct-adapter__images' in sn.attrib['class'] or sn.xpath('.//div[contains(@class,"Images")]'):
             return True
 
         # игнорим видео
         if 't-construct-adapter__video' in sn.attrib['class']:
             return True
 
+        # игнорим видео
+        if sn.xpath('.//div[contains(@class,"video2_theme_online")]'):
+            return True
+
         # игонорим тизеры
         if sn.xpath('.//div[contains(@class,"teaser ")]'):
             return True
@@ -282,6 +433,14 @@ class YandexParser(object):
         if re.search(ur't-construct-adapter__.+-fact', sn.attrib['class'], re.I):
             return True
 
+        # счет матча
+        if sn.xpath('.//div[contains(@class,"sport-livescore")]'):
+            return True
+
+        # счет матча
+        if sn.xpath('.//div[contains(@class,"sport-tournament")]'):
+            return True
+
         # игнорим факты
         if sn.xpath('.//div[contains(@class,"fact")]'):
             return True
@@ -315,6 +474,10 @@ class YandexParser(object):
         if sn.xpath('.//div[contains(@class,"converter-form")]'):
             return True
 
+        # олимпиада
+        if sn.xpath('.//div[contains(@class,"olympiad")]'):
+            return True
+
         # калькулятор
         if 't-construct-adapter__calculator' in sn.attrib['class']:
             return True
@@ -343,11 +506,15 @@ class YandexParser(object):
         if 'card__colorize' in sn.attrib['class']:
             return True
 
-        html = etree.tostring(sn)
+        html = etree.tostring(sn, method='html', encoding='UTF-8')
         if 't-construct-adapter__market' in sn.attrib['class']:
             if re.search(ur'<div class="organic typo typo_text_m typo_line_s">\s*<div class="organic__content-wrapper clearfix">', html, re.I | re.M):
                 return True
 
+        # Яндекс.Путешествия
+        if u'<b>Яндекс.Путешествия</b>' in html:
+            return True
+
         # Вы нашли то, что искали(мобильная версия)
         if 'ugc_player_default' in html:
             return True
@@ -356,24 +523,51 @@ class YandexParser(object):
         if 'object-badge' in html:
             return True
 
-        # Подробное описание объекта
-        if 'entity-search' in html:
-            return True
-
         # калькулятор
         if 'calculator__wrapper' in html:
             return True
 
+        # чаты с организациями
+        if 'chat-list__header' in html:
+            return True
+
         # палитра
         if sn.xpath('.//div[contains(@class,"colorpalette")]'):
             return True
 
-        # Различные составные блоки
-        if sn.xpath('.//div[contains(@class,"composite_gap_")]'):
+        # исключаем блок директа с товарами
+        if sn.xpath('./div[contains(@class,"carousel")]'):
+            return True
+
+        # исключаем блок директа с компаниями
+        if sn.xpath('.//div[contains(@class,"companies-map-")]'):
+            return True
+
+        # спросите у консультантов
+        if sn.xpath('.//div[contains(@id,"BusinessChatCenter-")]'):
             return True
 
         return False
 
+    def _is_composite_gap_block(self, sn):
+        # Различные составные блоки
+        return sn.xpath('.//div[contains(@class,"composite_gap_")]')
+
+    def _need_exclude_composite_gap_block(self, composite_gaps, url):
+        if 'market.yandex.ru' in url and not self.exclude_market_yandex:
+            return False
+
+        if 'realty.yandex.ru' in url and not self.exclude_realty_yandex:
+            return False
+
+        if 'yandex.ru/maps' in url:
+            return False
+
+        if 'companies' in composite_gaps[0].attrib['class']:
+            return False
+
+        return True
+
     def get_snippets(self):
 
         try:
@@ -388,7 +582,18 @@ class YandexParser(object):
 
                 infected = 'template-infected' in sn.attrib['class']
 
-                title, url = self._get_title(sn, infected)
+                # различные составные блоки
+                is_composite_gap = self._is_composite_gap_block(sn)
+
+                try:
+                    title, url = self._get_title(sn, infected)
+                except YandexParserError:
+                    if is_composite_gap:
+                        continue
+                    raise
+
+                if is_composite_gap and self._need_exclude_composite_gap_block(is_composite_gap, url):
+                    continue
 
                 url, url_infected = self._get_true_url(sn, url)
                 infected |= url_infected
@@ -402,7 +607,7 @@ class YandexParser(object):
                     'domain': domain,
                     'p': position,
                     'u': url,
-                    'm': url.startswith('http://maps.yandex.ru'),
+                    'm': url.startswith('https://maps.yandex.ru'),
                     't': None,  # title snippet
                     's': None,  # body snippet
                     'i': infected,
@@ -443,6 +648,9 @@ class YandexParser(object):
         # if pagecount <= 50:
         #     return snippets
 
+        if not self.exclude_market_yandex or not self.exclude_realty_yandex:
+            return snippets
+
         len_snippets = len(snippets)
         if len_snippets % 10 == 0:
             return snippets
@@ -484,8 +692,21 @@ class YandexParser(object):
         if 'checkcaptcha' not in self.content:
             return
 
-        match_captcha = self.patterns['captcha'].findall(self.content)
+        patterns = [
+            self.patterns['captcha'],
+            re.compile(u'<div class="captcha__image"><img\s*src=\"([^\"]+)\"'),
+        ]
+
+        match_captcha = None
+        for pattern in patterns:
+            match_captcha = pattern.findall(self.content)
+            if match_captcha:
+                break
+
+        if not match_captcha:
+            raise YandexParserError()
         url_image = match_captcha[0]
+
         html = lxml.html.fromstring(self.content)
         form = html.forms[0]
         form_data = dict(form.form_values())
@@ -514,7 +735,7 @@ class YandexSerpCleaner(object):
         ur'ondblclick=".*?"',
         ur'style=".*?"',
         ur'<noscript>.*?<\/noscript>',
-        ur'<link.*?/>',
+        ur'<link.*?/?>',
         ur'<!--.*?-->',
         ur'<i\s+><\/i>',
         ur'data-bem\s*=\s*"[^"]*?"',
